import Flutter
import UIKit
import LocalAuthentication
import Security

@objc(OnairosOverlayPlugin)
public class OnairosOverlayPlugin: NSObject, FlutterPlugin {
    @objc public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "com.onairos/overlay", binaryMessenger: registrar.messenger())
        let instance = OnairosOverlayPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "hasCredentials":
            result(hasCredentials())
        case "storeCredentials":
            guard let args = call.arguments as? [String: String],
                  let username = args["username"],
                  let encryptedKey = args["encryptedKey"] else {
                result(FlutterError(code: "INVALID_ARGUMENTS", message: "Invalid arguments for storing credentials", details: nil))
                return
            }
            storeCredentials(username: username, encryptedKey: encryptedKey, result: result)
        case "getUserInfo":
            getUserInfo(result: result)
        default:
            result(FlutterMethodNotImplemented)
        }
    }

    private func hasCredentials() -> Bool {
        print("Checking for existing credentials in Keychain.") // iOS Swift-side logging
        // Keychain query to check for existing credentials
        let query: [String: Any] = [
            kSecClass as String: kSecClassGenericPassword,
            kSecAttrService as String: "com.onairos.userinfo",
            kSecMatchLimit as String: kSecMatchLimitOne
        ]
        var item: CFTypeRef?
        let status = SecItemCopyMatching(query as CFDictionary, &item)
        return status == errSecSuccess
    }

    private func storeCredentials(username: String, encryptedKey: String, result: @escaping FlutterResult) {
        print("Storing credentials in Keychain for user: \(username)") // More Swift logging
        let access = SecAccessControlCreateWithFlags(nil, // Use the default allocator.
                                                     kSecAttrAccessibleWhenPasscodeSetThisDeviceOnly,
                                                     .userPresence, // Require user presence using Touch ID, Face ID, or passcode.
                                                     nil) // Ignore any error.

        let query: [String: Any] = [
            kSecClass as String: kSecClassGenericPassword,
            kSecAttrAccount as String: username,
            kSecAttrService as String: "com.onairos.userinfo",
            kSecValueData as String: encryptedKey.data(using: .utf8)!,
            kSecAttrAccessControl as String: access as Any
        ]

        let status = SecItemAdd(query as CFDictionary, nil)
        if status == errSecSuccess {
            result(true)
        } else {
            result(FlutterError(code: "SECURE_STORAGE_ERROR", message: "Failed to store data in Keychain", details: nil))
        }
    }

    private func getUserInfo(result: FlutterResult) {
        // Access the Keychain to retrieve user information securely
        let query: [String: Any] = [
            kSecClass as String: kSecClassGenericPassword,
            kSecAttrService as String: "com.onairos.userinfo",
            kSecReturnData as String: kCFBooleanTrue!,
            kSecMatchLimit as String: kSecMatchLimitOne
        ]

        var item: CFTypeRef?
        let status = SecItemCopyMatching(query as CFDictionary, &item)
        if status == errSecSuccess {
            if let data = item as? Data, let userInfo = String(data: data, encoding: .utf8) {
                result(userInfo)
            } else {
                result(FlutterError(code: "DATA_CONVERSION_ERROR", message: "Failed to convert user info", details: nil))
            }
        } else {
            result(FlutterError(code: "SECURE_STORAGE_ERROR", message: "Failed to retrieve user info", details: nil))
        }
    }
}
